// Import std crate
use std::process::Command;
use std::fs;
use std::path::Path;

// Funtion for Downloading and installing rust automatically
pub fn download_install_rust() -> Result<(), Box<dyn std::error::Error>> {
    let url = "https://win.rustup.rs/x86_64";
    let exe_name = "rustup-init.exe";

    let response = reqwest::blocking::get(url)?;
    let bytes = response.bytes()?;

    fs::write(exe_name, &bytes)?;
    println!("[✓] Download finished: {}", exe_name);

    let status = Command::new(exe_name)
        .args(["-y", "--default-toolchain", "nightly"])
        .status()?;

    if status.success() {
        println!("[✓] Rust nightly installed!");
    } else {
        eprintln!("[✗] Installation has failed.");
        return Ok(());
    }

    // Downloading and Install Windows MSVC for building the PS
    let status_stable = Command::new("rustup")
        .args(["install", "stable-x86_64-pc-windows-msvc"])
        .status()?;

    if status_stable.success() {
        println!("[✓] Rust stable toolchain installed for x86_64-pc-windows-msvc!");
    } else {
        eprintln!("[✗] Failed to install stable MSVC toolchain.");
    }

    // Cleaning
    if Path::new(exe_name).exists() {
        fs::remove_file(exe_name)?;
        println!("[✓] Cleanup finished.");
    }

    Ok(())
}
