// Import "std crate" with necessary depedencies for buttons, "Eframe" for UI, And everything needed for the buttons funtions

use std::collections::HashMap;
use std::sync::{Arc, Mutex};

use eframe::egui;
use crate::core::{rustdl, zenless::zenless_client_patch, zenless::zenless_srv};


// Define the Launcher structures and functionnalities
pub struct ZZZRRLauncher {
    selected_tab: Tab,
    available_servers: Vec<String>,
    logs: Arc<Mutex<HashMap<String, Vec<String>>>>,
}

#[derive(PartialEq)]
enum Tab {
    Download,
    Server,
    Zzz,
    ServerInstance(String),
}

impl Default for ZZZRRLauncher {
    fn default() -> Self {
        let logs = Arc::new(Mutex::new(HashMap::new()));
        let available_servers = zenless_srv::list_available_servers();

        Self {
            selected_tab: Tab::Download,
            available_servers,
            logs,
        }
    }
}


// Generate an UI with some buttons, each buttons are different
impl eframe::App for ZZZRRLauncher {
    fn update(&mut self, ctx: &egui::Context, _frame: &mut eframe::Frame) {
        egui::TopBottomPanel::top("top_panel").show(ctx, |ui| {
            ui.horizontal(|ui| {
                if ui.selectable_label(self.selected_tab == Tab::Download, "📥 Download").clicked() {
                    self.selected_tab = Tab::Download;
                }
                if ui.selectable_label(self.selected_tab == Tab::Server, "🛠 Server").clicked() {
                    self.selected_tab = Tab::Server;
                }
                if ui.selectable_label(self.selected_tab == Tab::Zzz, "🎮 ZZZ").clicked() {
                    self.selected_tab = Tab::Zzz;
                }

                for server in &self.available_servers {
                    if ui
                        .selectable_label(
                            self.selected_tab == Tab::ServerInstance(server.clone()),
                            format!("🔌 {}", server),
                        )
                        .clicked()
                    {
                        self.selected_tab = Tab::ServerInstance(server.clone());
                    }
                }
            });
        });

        egui::CentralPanel::default().show(ctx, |ui| {
            match &self.selected_tab {
                Tab::Download => {
                    ui.heading("Download Tools");
                    if ui.button("Download Server").clicked() {
                        zenless_srv::zzz_download_server();
                        self.available_servers = zenless_srv::list_available_servers();
                    }
                    if ui.button("Download Client Patch").clicked() {
                        zenless_client_patch::zzz_download_client_patch();
                    }
                    if ui.button("Download Rust Nightly").clicked() {
                        let _ = rustdl::download_install_rust();
                    }
                }
                Tab::Server => {
                    ui.heading("Server Management");
                    if ui.button("Build Server").clicked() {
                        zenless_srv::zzz_build_server();
                        self.available_servers = zenless_srv::list_available_servers();
                    }

                    ui.separator();
                    for server in &self.available_servers {
                        if ui.button(format!("🚀 Launch {}", server)).clicked() {
                            zenless_srv::launch_server_with_logs(server, Arc::clone(&self.logs));
                            self.selected_tab = Tab::ServerInstance(server.clone());
                        }
                    }
                }
                Tab::Zzz => {
                    ui.heading("ZZZ Client Patch");
                    if ui.button("Build Client Patch").clicked() {
                        zenless_client_patch::zzz_build_client_patch();
                    }
                    if ui.button("Launch ZZZ").clicked() {
                        zenless_client_patch::zzz_client_patch_loaded();
                    }
                }
                Tab::ServerInstance(server_name) => {
                    ui.heading(format!("📡 Logs for {}", server_name));
                    if let Some(lines) = self.logs.lock().unwrap().get(server_name) {
                        egui::ScrollArea::vertical().show(ui, |ui| {
                            for line in lines.iter().rev().take(100).rev() {
                                ui.monospace(line);
                            }
                        });
                    } else {
                        ui.label("No logs yet.");
                    }
                }
            }
        });
    }
}
